#!/bin/sh
#
#  xruntrace1_watch.sh
#
#  Real-Time Preemption, -RT-2.6.10-rc2-mm2-V0.7.31-13
#
#  User-triggerable ALSA xrun tracing patch: if a sound IO channel has
#  xrun_debug enabled in /proc then user_trace_stop() will be called
#  before printing the xrun message, and the current trace will be saved
#  to /proc/latency_trace. This is a 'one-shot' tracing method for now.

[ ${UID} -ne 0 ] && echo "`basename $0`: Must be root, sorry." && exit 1

TRACE_MAX=$1

[ -z "${TRACE_MAX}" ] && TRACE_MAX=0

# Make sure we have anything needed... 
make all > /dev/null || exit 2

show_info ()
{
   CMD="$*" 
   echo 
   echo "--"
   echo "# ${CMD}"
   echo
   ${CMD}
}

show_info "uname -a"

show_info "cat /proc/asound/version"
show_info "cat /proc/asound/cards"

show_info "cat /proc/interrupts"
show_info "ps -eo pid,class,rtprio,pri,ni,pcpu,stat,comm --sort -rtprio" | head -23

show_info "./xruntrace1.sh status"

# Prepare the ground...
dmesg -c > /dev/null 2>&1

# Prevent leaving the house dirty...
trap './xruntrace1.sh stop; exit 3' INT TERM

# Enter watch loop...
show_info "./xruntrace1.sh start"
LAST=`cat /proc/sys/kernel/preempt_max_latency`

TRACE_NO=0
while [ ${TRACE_NO} -le ${TRACE_MAX} ]
do
    CURR=`cat /proc/sys/kernel/preempt_max_latency`
    if [ "${CURR}" != "${LAST}" ]; then
        # Dump the trace and most recent dmesg...
        TRCFILE="xruntrace1-`uname -r`-`date +"%Y%m%d%H%M%S"`.trc"
        echo "=======================================================" >> ${TRCFILE}
        echo "XRUN latency trace dump    `date`"                       >> ${TRCFILE}
        echo "=======================================================" >> ${TRCFILE}
        echo "# dmesg -c"                                              >> ${TRCFILE}
        echo                                                           >> ${TRCFILE}
        dmesg -c | tee -a ${TRCFILE}
        echo                                                           >> ${TRCFILE}
        echo "- - - - - - - - - - - - - - - - - - - - - - - - - - - -" >> ${TRCFILE}
        cat /proc/latency_trace                                        >> ${TRCFILE}
        echo                                                           >> ${TRCFILE}
        sync
        show_info "./xruntrace1.sh stop"
        gzip ${TRCFILE}
        # Prepare for the next one...
        show_info "./xruntrace1.sh start"
        LAST=`cat /proc/sys/kernel/preempt_max_latency`
        TRACE_NO=$((${TRACE_NO} + 1))
    else
        sleep 1
    fi
done

show_info "./xruntrace1.sh stop"

exit 0
